@@ -2,18 +2,25 @@ attach(NULL, name = "CheckExEnv")
 assign(".CheckExEnv", as.environment(2), pos = length(search())) # base
 ## This plot.new() patch has no effect yet for persp();
 ## layout() & filled.contour() are now ok
-assign("plot.new", function() { .Internal(plot.new())
-		       pp <- par(c("mfg","mfcol","oma","mar"))
-		       if(all(pp$mfg[1:2] == c(1, pp$mfcol[2]))) {
-			 outer <- (oma4 <- pp$oma[4]) > 0; mar4 <- pp$mar[4]
-			 mtext(paste("help(",..nameEx,")"), side = 4,
-			       line = if(outer)max(1, oma4 - 1) else min(1, mar4 - 1),
-			       outer = outer, adj=1, cex= .8, col="orchid")} },
+assign("plot.new",
+       function() {
+	   .Internal(plot.new())
+	   pp <- par(c("mfg","mfcol","oma","mar"))
+	   if(all(pp$mfg[1:2] == c(1, pp$mfcol[2]))) {
+               outer <- (oma4 <- pp$oma[4]) > 0; mar4 <- pp$mar[4]
+               mtext(paste("help(", ..nameEx, ")"), side = 4,
+                     line = if(outer)max(1, oma4 - 1) else min(1, mar4 - 1),
+                     outer = outer, adj = 1, cex = .8, col = "orchid")
+	   }
+       },
        env = .CheckExEnv)
-assign("cleanEx", function(env = .GlobalEnv) {
-	rm(list = ls(envir = env, all.names = TRUE), envir = env)
-	RNGkind("Wichmann-Hill", "default")
-	assign(".Random.seed", c(0,rep(7654,3)), pos=1)
+assign("cleanEx",
+       function(env = .GlobalEnv) {
+	   rm(list = ls(envir = env, all.names = TRUE), envir = env)
+           RNGkind("Wichmann-Hill", "default")
+	   assign(".Random.seed", c(0, rep(7654, 3)), pos = 1)
+	   assign("T", NULL, pos = 1);
+	   assign("F", NULL, pos = 1);
        },
        env = .CheckExEnv)
 assign("..nameEx", "__{must remake R-ex/*.R}__", env = .CheckExEnv) #-- for now
@@ -37,7 +44,7 @@ cleanEx(); ..nameEx <- "dperm"
 
 x <- c(0.5, 0.5, 0.6, 0.6, 0.7, 0.8, 0.9)
 y <- c(0.5, 1.0, 1.2, 1.2, 1.4, 1.5, 1.9, 2.0)
-r <- rank(c(x,y))
+r <- cscores(c(x,y), type="Wilcoxon")
 pperm(sum(r[seq(along=x)]), r, 7)
 
 # Compare the exact algorithm as implemented in ctest and the
@@ -48,7 +55,7 @@ pperm(sum(r[seq(along=x)]), r, 7)
 n <- 10
 x <- rnorm(n, 2)
 y <- rnorm(n, 3)
-r <- rank(c(x,y))
+r <- cscores(c(x,y), type="Wilcoxon")
 
 # exact distribution using Streitberg-Roehmel
 
@@ -71,7 +78,7 @@ plot(dw, dwexac, main="Wilcoxon", xlab="dwilcox", ylab="dperm")
 n <- 10
 x <- rnorm(n, 5)
 y <- rnorm(n, 5)
-r <- rank(abs(x - y))
+r <- cscores(abs(x - y), type="Wilcoxon")
 pperm(sum(r[x - y > 0]), r, length(r))
 wilcox.test(x,y, paired=TRUE, alternative="less")
 psignrank(sum(r[x - y > 0]), length(r))
@@ -84,8 +91,7 @@ y <- rnorm(n, 2, 2)
 
 # exact distribution using Streitberg-Roehmel
 
-r <- rank(c(x,y))
-sc <- pmin(r, 2*n - r +1)
+sc <- cscores(c(x,y), type="Ansari")
 dabexac <- dperm(0:(n*(2*n+1)/2), sc, n)
 sum(dabexac)
 tr <- which(dabexac > 0)
@@ -104,9 +110,7 @@ plot(dab[tr], dabexac[tr], main="Ansari", xlab="dansari", ylab="dperm")
 n <- 10
 x <- rnorm(n)
 y <- rnorm(n)
-N <- length(x) + length(y)
-r <- rank(c(x,y))
-scores <- qnorm(r/(N+1))
+scores <- cscores(c(x,y), type="NormalQuantile")
 X <- sum(scores[seq(along=x)])  # <- v.d. Waerden normal quantile statistic
 
 # critical value, two-sided test
@@ -120,8 +124,7 @@ p1 <- pperm(X, scores, length(x), alternative="two.sided")
 # generate integer valued scores with the same shape as normal quantile
 # scores, this no longer v.d.Waerden, but something very similar
 
-scores <- scores - min(scores)
-scores <- round(scores*N/max(scores))
+scores <- cscores(c(x,y), type="NormalQuantile", int=TRUE)
 
 X <- sum(scores[seq(along=x)])
 p2 <- pperm(X, scores, length(x), alternative="two.sided")
@@ -138,124 +141,20 @@ contr <- c(80, 94, 85, 90, 90, 90, 108, 94, 78, 105, 88)
 # compute the v.d. Waerden test and compare the results to StatXact-4 for
 # Windows:
 
-r <- rank(c(contr, treat))
-sc <- qnorm(r/16)
+sc <- cscores(c(contr, treat), type="NormalQuantile")
 X <- sum(sc[seq(along=contr)])
 round(pperm(X, sc, 11), 4)      # == 0.0462 (StatXact)
 round(pperm(X, sc, 11, alternative="two.sided"), 4)     # == 0.0799 (StatXact)
 
 # the alternative method returns:
 
-sc <- sc - min(sc)
-sc <- round(sc*16/max(sc))
+sc <- cscores(c(contr, treat), type="NormalQuantile", int=TRUE)
 X <- sum(sc[seq(along=contr)])
 
 round(pperm(X, sc, 11), 4)      # compare to 0.0462 
 round(pperm(X, sc, 11, alternative="two.sided"), 4)     # compare to 0.0799
 
 
-
-# paired observations
-
-hansi <- c()
-seppl <- c()
-for (i in 1:10)
-{
-        m <- sample(10:50, 1)
-        score <- sample(m)
-        val <- sample(0:m, 1)
-        # cat("m: ", m, "n: ", n, " val: ", val, "\n")
-        hansi <- c(hansi,  psignrank(val, m))
-        cat("psignrank: ", hansi[length(hansi)])
-        seppl <- c(seppl, pperm(val, score, m, alternative="less"))
-        cat(" pperm: ", seppl[length(seppl)], "\n")
-}
-
-cat("Max difference: ", max(abs(hansi - seppl)), "\n")
-
-
-  stopifnot(max(abs(hansi - seppl)) <= 1e-10)
-
-
-hansi <- c()
-seppl <- c()
-for (i in 1:10)
-{
-        m <- sample(10:50, 1)
-        score <- sample(m)
-        prob <- runif(1)
-        # cat("m: ", m, "n: ", n, " prob: ", prob, "\n")
-        hansi <- c(hansi,  qsignrank(prob, m))
-        cat("qwilcox: ", hansi[length(hansi)])
-        seppl <- c(seppl, qperm(prob, score, m))
-        cat(" qperm: ", seppl[length(seppl)], "\n")
-}
-
-cat("Max difference: ", max(abs(hansi - seppl)), "\n")
-
-
-  stopifnot(max(abs(hansi - seppl)) <= 1e-10)
-
-
-# independent observations
-
-hansi <- c()
-seppl <- c()
-for (i in 1:10)
-{
-        m <- sample(10:50, 1)
-        if (runif(1) < 0.5)
-                n <- sample(10:50, 1)
-        else    
-                n <- m
-        score <- sample(n+m)
-        val <- sample(0:(m*n), 1)
-        # cat("m: ", m, "n: ", n, " val: ", val, "\n")
-        hansi <- c(hansi,  pwilcox(val, m, n))
-        cat("pwilcox: ", hansi[length(hansi)])
-        seppl <- c(seppl, pperm(val + m*(m+1)/2, score, m,
-alternative="less"))
-        cat(" pperm: ", seppl[length(seppl)], "\n")
-}
-
-cat("Max difference: ", max(abs(hansi - seppl)), "\n")
-
-
-  stopifnot(max(abs(hansi - seppl)) <= 1e-10) 
-
-
-hansi <- c()
-seppl <- c()
-for (i in 1:10)
-{
-        m <- sample(10:50, 1)
-        if (runif(1) < 0.5)
-                n <- sample(10:50, 1)
-        else
-                n <- m
-        score <- sample(n+m)
-        prob <- runif(1)
-        # cat("m: ", m, "n: ", n, " prob: ", prob, "\n")
-        hansi <- c(hansi,  qwilcox(prob, m, n))
-        cat("qwilcox: ", hansi[length(hansi)])
-        seppl <- c(seppl, qperm(prob, score, m) - m*(m+1)/2)
-        cat(" qperm: ", seppl[length(seppl)], "\n")
-}
-
-cat("Max difference: ", max(abs(hansi - seppl)), "\n")
-
-
-  stopifnot(max(abs(hansi - seppl)) <= 1e-10) 
-
-
-
-  # bugged me
-  # stopifnot(pperm(36, c(16,15,5,11,14), 5, alternative="t") == 0.625)
-
-
-
-
-
 ## Keywords: 'distribution'.
 
 
@@ -312,16 +211,24 @@ pt
 
   stopifnot(round(pt$p.value, 4) == 0.0564)
 
+
+pt <- perm.test(treat, contr, exact=FALSE)
+pt
+
+  stopifnot(round(pt$p.value, 4) == 0.1070)
+
+
 # one-sample AIDS data (differences only), page 179
 
 diff <- c(-149, 51, 0, 126, -106, -20, 0, -52, -292, 0, -103, 0, -84, -89,
 -159, -404, -500, -259, -14, -2600)
 
-# p-values in StatXact == 0.0011 one-sided, 0.0021 two.sided are slightly
-# different, STATISTIC is ok ?!?
+# p-values in StatXact == 0.0011 one-sided, 0.0021 two.sided 
 
 perm.test(diff)
 
+perm.test(diff, alternative="less")
+
 pt <- perm.test(diff, exact=FALSE)
 
   # StatXact page 179
@@ -542,6 +449,30 @@ stopifnot(wilcox.test(x,conf.int=TRUE)$estimate ==
           wilcox.exact(x,conf.int=TRUE)$estimate)
 
 
+# Table 9.19 StaXact-4 manual: lung cancer clinical trial
+time <- c(257, 476, 355, 1779, 355, 191, 563, 242, 285, 16, 16, 16, 257, 16)
+cens <- c(0,0,1,1,0,1,1,1,1,1,1,1,1,1)
+
+# round logrank scores
+scores <- cscores.Surv(cbind(time, cens))
+T <- sum(scores[1:5])
+prob <- pperm(T, scores, m=5, al="le")
+prob
+stopifnot(all.equal(round(prob, 3), 0.001))
+prob <- pperm(T, scores, m=5, al="tw")
+prob
+stopifnot(all.equal(round(prob, 3), 0.001))
+
+
+# map into integers
+scores <- cscores.Surv(cbind(time, cens), int=TRUE)
+T <- sum(scores[1:5])
+prob <- pperm(T, scores, m=5, al="le")
+prob
+stopifnot(all.equal(round(prob, 3), 0.001))
+prob <- pperm(T, scores, m=5, al="tw")
+prob
+stopifnot(all.equal(round(prob, 3), 0.001))
 
 ## Keywords: 'htest'.
 
